# GyriGauge: evaluate your segmentation

## Introduction

With an increasing amount of model predictions the necessity for quality measures grows since the evaluation of predictions is crucial.
In this package, we want to provide the functionality to evaluate segmentation quality.
Focused on brain MR imaging, this code basis was tested extensively.

## Description
For segmentation quality evaluation, we calculate metrics based on three different types of information with many metrics included:
- information theory
    - contingency table
    - marginal entropy
    - joint entropy
    - mutual information
    - variation of information
- spatial distance of point clouds
    - (percentile) hausdorff distance with different distance functions. Note that this is the distance in voxels based on the grid.
- spatial overlap of specific regions.
    - accuracy
    - dice dissimilarity
    - continous dice
    - generalized dice
    - intersection over union
    - boundary intersection over union

Feel free to experiment with them.

## Visuals
**TODO** Include nice image of what metrics to and how input should look like

## Project setup
The code is designed to run on GPU but can be run on CPU in principle. Please consider that computational time increases.

### Hardware requirements
The code has been tested with 3D tensors of size `[352,352,352]` and integer values in range 0-5. Hardware in use was
- GPU RAM: `7.0 GB`

### Software requirements
The main package requires `numpy` and `pytorch`. `pytorch` needs to be installed depending on the hardware in use. It is tested with versions `python=3.10`, `numpy=1.26.4`, and `pytorch=2.1.2` with GPU `cuda=12.1` and `cudnn=8.9.2`.

### Installation
The easiest way of getting this run, install it in "editable" mode by cloning the repository or using it as git submodule:
```bash
cd <path_to_your_repository_src>
git submodule add https://gitlab.tuebingen.mpg.de/jsteiglechner/gyrigauge gyrigauge
git submodule update --init gyrigauge
git submodule update --remote gyrigauge
```

### Experiences
This code has been tested with
- OS Ubuntu 24
- CPU: Intel(R) Xeon(R) W-2255 CPU @ 3.70GHz
- GPU NVIDIA RTX A4000

We tested 3D tensors of size `[352,352,352]` and integer values in range 0-5 with important types 1-4 of 10 brain MRI segmentations by running standard `evaluate_prediction`. Here we present mean results:
| Metric          | Object of interest | Result     |
| --------------- | ------------------ | ---------- |
| IoU             | time               | `0.025 s`  |
| IoU             | memory             | `3.608 GB` |
| boundary IoU    | time               | `0.132 s`  |
| boundary IoU    | memory             | `6.234 GB` |
| 95th-hausdorff  | time               | `346.8 s`  |
| 95th-hausdorff  | memory             | `5.346 GB` |
| VoI             | time               | `0.976 s`  |
| VoI             | memory             | `1.930 GB` |

## Basic usage
To evaluate most important metrics (IoU, boundary IoU, 95th percentile hausdorff distance, voi) of a prepared segmentation tensor `prepared_output` with its reference `target_tensor` containing 3 types but only want to calculate without background:
```python

from evaluate_segmentation_quality import evaluate_prediction

evaluate_prediction(
    output=prepared_output,
    target=target_tensor,
    predicted_types={0:'bg', 1:'left', 2:'right'},
    considered_types=[1,2],
)
```
The expected output is a dictionary from metric labels to values.

## Support and contributing
Having trouble or wanting to contribute? Do not hesitate to contact [Julius](mailto:julius.steiglechner@tuebingen.mpg.de). If not response in 7 days, contact him again.

## Authors and acknowledgment
- Julius
- Lucas
- FLEXseg

## License
This project is licensed under the [MIT License](https://gitlab.tuebingen.mpg.de/jsteiglechner/gyrigauge/-/blob/main/LICENSE).

## Project status
There are open tasks:
- fix information theory metrics. They give nan sometimes.
- Write tests
- Include metrics that are important for loss calculation during training.

Project development has slowed down because the main functionality that was needed is reached.
